<?php
/*
 **************************************************************************
 *  (c) Copyright IBM Corporation. 2007 , 2009.  All Rights Reserved
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 **************************************************************************
 */


require_once ( "Diagram.php" );

/**
 * Renders the Statement interator tree
 *
 */
class XTree
{
    public $idsadmin;


    /**
     * Class constructor
     */

    function __construct ( &$idsadmin, $sid = 0, $path )
    {
        $this->idsadmin = $idsadmin;
        $this->initialize ( $sid, $path );
    }

    protected function initialize ( $sid, $path )
    {
        /*
         * Retrieve the iterators associated
         * with the given statement ID
         */

        $iterators = $this->getIterators($sid);

        /*
         * Reconstruct the iterator tree in XML
         */

        $xml = $this->generateXML($iterators);

        /*
         * Render the iterator tree
         */

        $diagram = new Diagram ( );


        $diagram->render ( $xml, $path );
         

    }

    protected function generateXML($iterators)
    {
        reset($iterators);
        $root = current($iterators);

        $xml  = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
        $xml .= "<diagram>";
        $xml .= $this->generateXMLForIterator($iterators, $root);
        $xml .= "</diagram>";

        return $xml;
    }

    protected function generateXMLForIterator($iterators, $iterator)
    {
        /*
         * Generate a node tag for this iterator
         */
         
        $xml = "<node "
        . "title=\""
        . $iterator->getID ( )
        . "."
        . trim ( $iterator->getInfo ( ) )
        . "\" "
        . ">" ;

        /*
         * Generate node data for this iterator
         */

        /*
         * Generate node data for this iterator
         */
        $xml .= sprintf ( "%25s\n"
        , $this->getFormatedTableNames($iterator->getTableName(),25) );
        $xml .= sprintf ( "%-14s %10d\n"
        , "Cost"
        , $iterator->getCost ( ) );
        $xml .= sprintf ( "%-14s %10d\n"
        , "Rows estimated"
        , $iterator->getRowsEstimated ( ) );
        $xml .= sprintf ( "%-14s %10d\n"
        , "Rows processed"
        , $iterator->getRowsProcessed ( ) );
        $xml .= sprintf ( "%-14s %10.4f\n"
        , "Elapsed Time"
        , $iterator->getTime ( ) );


        /*
         * If the iterator has a child
         * to its left and/or right, recurse
         */

        if ($iterator->isNode())
        {
            if ($iterator->hasLeft())
            {
                $left = $iterators[$iterator->getLeft()];
                $xml .= $this->generateXMLForIterator($iterators, $left);
            }
            if ($iterator->hasRight())
            {
                $right = $iterators[$iterator->getRight()];
                $xml .= $this->generateXMLForIterator($iterators, $right);
            }
        }

        /*
         * Delimit the node tag, and return the XML
         */

        $xml .= "</node>";
        return $xml;
    }

    protected function getIterators($sid)
    {
        $iter_table = "sysmaster:syssqltrace_iter";
        if ( $this->idsadmin->in['act'] == 'sqltrace' )
        {
            $mode = isset( $this->idsadmin->in['mode'] ) ? $this->idsadmin->in['mode'] : 1;
            if ( $mode == 2 )
            {
                $iter_table = "sysadmin:mon_syssqltrace_iter";
            }
        }
        $db = $this->idsadmin->get_database("sysmaster");

        $query = " SELECT                 "
        . " sql_id         ,       "
        . " sql_itr_id     ,       "
        . " sql_itr_left   ,       "
        . " sql_itr_right  ,       "
        . " sql_itr_cost   ,       "
        . " sql_itr_estrows,       "
        . " sql_itr_numrows,       "
        . " sql_itr_type   ,       "
        . " sql_itr_misc   ,       "
        . " sql_itr_info   ,       ";

        if (  Feature::isAvailable ( Feature::CHEETAH2, $this->idsadmin->phpsession->serverInfo->getVersion()  )  )
        {
            $query .= "sql_itr_time ,           " .
        	   			 "sql_itr_partnum          " ;

        } else {
        	   $query .= "0 AS sql_itr_time ,      " .
        	   		 	 "0 AS sql_itr_partnum     " ;
        }
         
        $query .= " FROM  {$iter_table} " .
              	  " WHERE sql_id =         " . $sid .
        		  " ORDER BY sql_itr_id    ";
//error_log("XTREE");
        //error_log($query);
        $statement = $db->query($query);

        $iterators = array();
        while ($result = $statement->fetch())
        {

            $iterators[$result['SQL_ITR_ID']] =
            new XTreeIterator($this->idsadmin,
            $result['SQL_ID'         ],
            $result['SQL_ITR_ID'     ],
            $result['SQL_ITR_LEFT'   ],
            $result['SQL_ITR_RIGHT'  ],
            $result['SQL_ITR_COST'   ],
            $result['SQL_ITR_ESTROWS'],
            $result['SQL_ITR_NUMROWS'],
            $result['SQL_ITR_TYPE'   ],
            $result['SQL_ITR_MISC'   ],
            $result['SQL_ITR_INFO'   ],
            $result['SQL_ITR_TIME'   ],
            $result['SQL_ITR_PARTNUM'] );
        }

        return $iterators;
    }

    protected function getFormatedTableNames($tabname, $nspaces=25) {
        $len = strlen( $tabname );
        if ( $len < 1 )
        return "-------------------------";
        $rem = $nspaces - 6;
        if ( $len > $rem )	{
            $format = "Table %" . $rem . "s";
            return sprintf( $format, $tabname);
        }
        /*
         * The table name is short than the space provided
         */
       	$format = "Table %" . $rem . "s";
       	return sprintf( $format,$tabname);
    }
     
} # End of class XTree

/**
 * Keeps track of each iterator in the tree
 *
 */
class XTreeIterator
{
    protected $idsadmin;
    protected $sid;
    protected $id;
    protected $left;
    protected $right;
    protected $cost;
    protected $estrows;
    protected $numrows;
    protected $type;
    protected $misc;
    protected $info;
    protected $time;
    protected $partnum;

    const IT_NOP      =  0; // exists only for tutorial purposes!
    const IT_SCAN     =  1; // sequential or indexed table/file scan
    const IT_CLSCAN   =  2; // scan of a collection variable
    const IT_ITERSCAN =  3; // scan of a iterator udr
    const IT_SORT     =  4; // produce sorted output from random input
    const IT_RDUPS    =  5; // remove duplicates
    const IT_MERGE    =  6; // two-way (for now) merge iterator
    const IT_JOIN     =  7; // Nested Loop Join (incl. index and cart. prod.)
    const IT_MJOIN    =  8; // sort/merge join
    const IT_HJOIN    =  9; // dynamic hash join
    const IT_GROUP    = 10; // provides group-by and aggrgation
    const IT_XCHG     = 11; // data exchange
    const IT_REMOTE   = 12; // Remote [Scan] iterator
    const IT_STREAM   = 13; // Stream iterator
    const IT_SOW      = 14; // Set Oriented Write iterator
    const IT_ULSTRM   = 15; // Unload stream iterator
    const IT_INSERT   = 16; // Insert iterator
    const IT_CLINSERT = 17; // insert in a collection variable
    const IT_LAST     = 18; // one too many!

    /*
     * Class constructor
     */

    function __construct($idsadmin,
    $sid     = 0,
    $id      = 0,
    $left    = 0,
    $right   = 0,
    $cost    = 0,
    $estrows = 0,
    $numrows = 0,
    $type    = 0,
    $misc    = 0,
    $info    = "",
    $time    = 0,
    $partnum = 0 )
    {
        $this->idsadmin = $idsadmin;
        $this->sid      = $sid;
        $this->id       = $id;
        $this->left     = $left;
        $this->right    = $right;
        $this->cost     = $cost;
        $this->estrows  = $estrows;
        $this->numrows  = $numrows;
        $this->type     = $type;
        $this->misc     = $misc;
        $this->info     = $info;
        $this->time     = $time;
        $this->partnum  = $partnum;
    }

    public function getCost         () { return $this->cost   ; }
    public function getInfo         () { return $this->info   ; }
    public function getID           () { return $this->id     ; }
    public function getLeft         () { return $this->left   ; }
    public function getMiscellaneous() { return $this->misc   ; }
    public function getRight        () { return $this->right  ; }
    public function getRowsEstimated() { return $this->estrows; }
    public function getRowsProcessed() { return $this->numrows; }
    public function getStatementID  () { return $this->sid    ; }
    public function getType         () { return $this->type   ; }
    public function getTime         () { return $this->time   ; }
    public function getPartnum      () { return $this->partnum; }
    public function getTableName    () {
        if ($this->getPartnum() == 0)
        return null;
        $db = $this->idsadmin->get_database("sysmaster");
        /*
         * Select the table name
         */
        $stmt = $db->query(	"SELECT lower(trim(tabname)) as tabname " .
        					" FROM systabnames WHERE partnum = " .
        $this->getPartnum()	);
         
        if ($res = $stmt->fetch())
        {
            return $res[ 'TABNAME' ];
        }
        return sprintf("0x%08X",$this->getPartnum());
    }

    public function hasLeft () { return(self::isValidID($this->left )); }
    public function hasRight() { return(self::isValidID($this->right)); }

    public function isLeaf() { return($this->isNode() == false); }
    public function isNode() { return($this->hasLeft() || $this->hasRight()); }

    public static function isValidID($id) { return ($id > 0); }
} // End of class XTreeIterator

/*
 abstract class AbstractIterator
 {
 }

 abstract class AbstractScanIterator
 extends AbstractIterator
 {
 }

 class IndexScanIterator
 extends AbstractScanIterator
 {
 }

 class SequentialScanIterator
 extends AbstractScanIterator
 {
 }
 */
?>
